
# q-expansion of newform 2352.2.a.l, downloaded from the LMFDB on 11 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2352
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [-1], [2], [0], [4], [-6], [-2], [-4], [-8], [-2], [0], [-10], [6], [4], [0], [6], [4], [-6], [-4], [-8], [-10], [0], [-4], [6], [14], [2], [8], [-12], [-2], [-14], [0], [-20], [10], [4], [6], [8], [10], [-20], [-8], [-22], [12], [18], [0], [2], [-10], [8], [-20], [-16], [12], [2], [-22], [0], [-2], [-12], [30], [24], [-22], [0], [-10], [26], [4], [-30], [28], [-8], [-10], [-18], [4], [18], [-12], [-22], [30], [8], [32], [22], [20], [-16], [-26], [-6], [18], [22], [-36], [6], [0], [-2], [-24], [4], [34], [10], [-22], [32], [28], [-16], [-8], [-12], [44], [24], [-6], [6], [20], [30], [12], [-2], [44], [-6], [-12], [-34], [-28], [-18], [-24], [-26], [48], [-42], [-22], [-44], [-8], [2], [-4], [24], [-18], [28], [2], [2], [18], [-12], [-4], [-2], [-10], [0], [-8], [-6], [12], [-24], [-48], [6], [22], [14], [2], [-36], [-6], [10], [-44], [38], [56], [36], [26], [56], [-14], [-42], [20], [32], [-2], [-18], [-20], [-24], [-12], [0], [-40], [-18], [22], [26], [-4], [26], [-8], [-12], [18], [-24], [16], [-14], [50], [-30], [-28], [-22], [24], [-54], [-48], [38], [36], [38], [-40], [10], [0], [-4], [22], [-10], [-64], [-30], [30], [44], [-54], [-48], [-34], [-12], [44], [-38], [60], [22], [50], [-50], [30], [-24], [-66], [48], [34], [30], [-12], [42], [32], [12], [-54], [-28], [50], [50], [-24], [4], [-24], [-26], [32], [-2], [-56], [62], [22], [40], [-30], [-16], [60], [38], [70], [-16], [-72], [-60], [30], [-20], [-16], [-54], [-76], [-48], [62], [6], [36], [40], [-68], [-28], [56], [-18], [-50], [-24], [-48], [-52], [4], [-48], [-2], [-66], [24], [-10], [-38], [-20], [-58], [-12], [2], [-74], [-64], [44], [-46], [26], [78], [28], [30], [38], [-28], [6], [-6], [28], [6], [64], [14], [-24], [52], [-66], [10], [-36], [0], [-24], [72], [82], [-12], [64], [18], [38], [56], [30], [30], [60], [-6], [-44], [14], [-6], [80], [-30], [-12], [-36], [38], [78], [-64], [-36], [52], [2], [-28], [10], [-8], [-58], [-48], [22], [34], [76], [-40], [22], [12], [-64], [-2], [18], [44], [58], [-54], [-16], [10], [-2], [-20], [4], [-80], [-58], [14], [-34], [0], [-4], [-76], [-92], [78], [-34], [-74], [16], [-10], [-6], [18], [72], [30], [12], [34], [4], [-48], [18], [92], [74], [30], [48], [22], [-26], [16], [-12], [-46], [-8], [-26], [22], [32], [4], [-20], [-22], [-54], [24], [-38], [28], [4], [-48], [70], [40], [-2], [60], [16], [-34], [-2], [-74], [90], [-86], [-40], [-30], [28], [-56], [-32], [-94], [68], [-16], [2], [66], [-44], [-52], [-40], [-6], [-32], [-26], [-12], [-10], [58], [-30], [32], [86], [18], [-88], [-50], [18], [44], [-4], [-82], [-10], [-44], [-36], [-22], [-86], [-32], [40], [30], [-8], [-34], [-14], [16], [84], [-26], [-70], [-100], [-102], [-12], [-88]]
